@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file driver.h
@@ -35,6 +39,11 @@
 #include "../MYODBC_ODBC.h"
 #include "installer.h"
 
+/* Disable _attribute__ on non-gcc compilers. */
+#if !defined(__attribute__) && !defined(__GNUC__)
+# define __attribute__(arg)
+#endif
+
 #ifdef APSTUDIO_READONLY_SYMBOLS
 #define WIN32	/* Hack for rc files */
 #endif
@@ -81,9 +90,9 @@ extern "C"
 # define FreeLibrary(module) dlclose((module))
 #endif
 
-#define ODBC_DRIVER	  "ODBC "MYODBC_STRSERIES" Driver"
-#define DRIVER_NAME	  "MySQL ODBC "MYODBC_STRSERIES" Driver"
-#define DRIVER_NONDSN_TAG "DRIVER={MySQL ODBC "MYODBC_STRSERIES" Driver}"
+#define ODBC_DRIVER	  "ODBC " MYODBC_STRSERIES " Driver"
+#define DRIVER_NAME	  "MySQL ODBC " MYODBC_STRSERIES " Driver"
+#define DRIVER_NONDSN_TAG "DRIVER={MySQL ODBC " MYODBC_STRSERIES " Driver}"
 
 #if defined(__APPLE__)
 
@@ -114,13 +123,8 @@ extern "C"
 /* Max Primary keys in a cursor * WHERE clause */
 #define MY_MAX_PK_PARTS 32
 
-#if MYSQL_VERSION_ID >= 50500
-# define x_free(A) { void *tmp= (A); if (tmp) my_free((char *) tmp); }
-# ifndef NEAR
-#  define NEAR 
-# endif
-#else
-# define x_free(A) { void *tmp= (A); if (tmp) my_free((char *) tmp,MYF(MY_WME+MY_FAE)); }
+#ifndef NEAR
+#define NEAR
 #endif
 
 /* We don't make any assumption about what the default may be. */
@@ -137,6 +141,10 @@ extern "C"
 # define CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR 2059
 #endif
 
+#ifndef SQL_PARAM_DATA_AVAILABLE
+# define SQL_PARAM_DATA_AVAILABLE 101
+#endif
+
 /* Connection flags to validate after the connection*/
 #define CHECK_AUTOCOMMIT_ON	1  /* AUTOCOMMIT_ON */
 #define CHECK_AUTOCOMMIT_OFF	2  /* AUTOCOMMIT_OFF */
@@ -180,12 +188,20 @@ typedef enum { DESC_HDR, DESC_REC } fld_loc;
 #define SQL_IS_LEN (-10)
 
 /* check if ARD record is a bound column */
-#define ARD_IS_BOUND(d) ((d)->data_ptr || (d)->octet_length_ptr)
+#define ARD_IS_BOUND(d) (d)&&((d)->data_ptr || (d)->octet_length_ptr)
 
 /* get the dbc from a descriptor */
 #define DESC_GET_DBC(X) (((X)->alloc_type == SQL_DESC_ALLOC_USER) ? \
                          (X)->exp.dbc : (X)->stmt->dbc)
 
+#define IS_BOOKMARK_VARIABLE(S) if (S->stmt_options.bookmarks != \
+                                    SQL_UB_VARIABLE) \
+  { \
+    set_stmt_error(stmt, "HY092", "Invalid attribute identifier", 0); \
+    return SQL_ERROR; \
+  }
+
+
 /* data-at-exec type */
 #define DAE_NORMAL 1 /* normal SQLExecute() */
 #define DAE_SETPOS_INSERT 2 /* SQLSetPos() insert */
@@ -218,14 +234,16 @@ typedef struct {
   SQLULEN      *bind_offset_ptr;
   SQLINTEGER    bind_type;
   SQLLEN        count;
+  SQLLEN        bookmark_count;
   /* Everywhere(http://msdn.microsoft.com/en-us/library/ms713560(VS.85).aspx
-     http://msdn.microsoft.com/en-us/library/ms712631(VS.85).aspx) I found 
+     http://msdn.microsoft.com/en-us/library/ms712631(VS.85).aspx) I found
      it's referred as SQLULEN* */
   SQLULEN      *rows_processed_ptr;
 
   /* internal fields */
   desc_desc_type  desc_type;
   desc_ref_type   ref_type;
+  DYNAMIC_ARRAY   bookmark;
   DYNAMIC_ARRAY   records;
   MYERROR         error;
   struct tagSTMT *stmt;
@@ -318,8 +336,12 @@ typedef struct stmt_options
   SQLUINTEGER      cursor_type;
   SQLUINTEGER      simulateCursor;
   SQLULEN          max_length, max_rows;
+  SQLULEN          query_timeout;
   SQLUSMALLINT    *rowStatusPtr_ex; /* set by SQLExtendedFetch */
-  my_bool      retrieve_data;
+  my_bool         retrieve_data;
+  SQLUINTEGER     bookmarks;
+  void            *bookmark_ptr;
+  my_bool         bookmark_insert;
 } STMT_OPTIONS;
 
 
@@ -331,7 +353,7 @@ typedef struct	tagENV
   LIST	       *connections;
   MYERROR      error;
 #ifdef THREAD
-  pthread_mutex_t lock;
+  myodbc_mutex_t lock;
 #endif
 } ENV;
 
@@ -355,9 +377,10 @@ typedef struct tagDBC
   int           txn_isolation;
   uint          port;
   uint          cursor_count;
+  ulong         net_buffer_len;
   uint          commit_flag;
 #ifdef THREAD
-  pthread_mutex_t lock;
+  myodbc_mutex_t lock;
 #endif
 
   my_bool       unicode;            /* Whether SQL*ConnectW was used */
@@ -367,6 +390,7 @@ typedef struct tagDBC
   DataSource    *ds;                /* data source used to connect (parsed or stored) */
   SQLULEN       sql_select_limit;   /* value of the sql_select_limit currently set for a session
                                        (SQLULEN)(-1) if wasn't set */
+  int           need_to_wakeup;      /* Connection have been put to the pool */
 } DBC;
 
 
@@ -388,6 +412,7 @@ typedef struct limit_scroller
 {
    char               *query, *offset_pos;
    unsigned int       row_count;
+   unsigned long long start_offset;
    unsigned long long next_offset, total_rows, query_len;
 
 } MY_LIMIT_SCROLLER;
@@ -409,6 +434,14 @@ typedef struct cursor
   MY_PK_COLUMN pkcol[MY_MAX_PK_PARTS];
 } MYCURSOR;
 
+enum OUT_PARAM_STATE
+{
+  OPS_UNKNOWN= 0,
+  OPS_BEING_FETCHED,
+  OPS_PREFETCHED,
+  OPS_STREAMS_PENDING
+};
+
 
 /* Main statement handler */
 
@@ -416,6 +449,7 @@ typedef struct tagSTMT
 {
   DBC               *dbc;
   MYSQL_RES         *result;
+  MEM_ROOT          alloc_root;
   my_bool           fake_result;
   MYSQL_ROW	        array,result_array,current_values;
   MYSQL_ROW	        (*fix_fields)(struct tagSTMT *stmt,MYSQL_ROW row);
@@ -461,6 +495,7 @@ typedef struct tagSTMT
   DESC *ird;
   DESC *apd;
   DESC *ipd;
+
   /* implicit descriptors */
   DESC *imp_ard;
   DESC *imp_apd;
@@ -474,8 +509,9 @@ typedef struct tagSTMT
 
   MY_LIMIT_SCROLLER scroller;
 
-  /* TODO: enum */
-  int out_params_state;
+  enum OUT_PARAM_STATE out_params_state;
+
+  int ssps_bind_result();
 } STMT;
 
 
@@ -485,7 +521,7 @@ extern uint decimal_point_length,thousands_sep_length;
 extern HINSTANCE NEAR s_hModule;  /* DLL handle. */
 #endif
 #ifdef THREAD
-extern pthread_mutex_t myodbc_lock;
+extern myodbc_mutex_t myodbc_lock;
 #endif
 
 /*
@@ -504,9 +540,18 @@ extern pthread_mutex_t myodbc_lock;
 # define MYSQL_TYPE_BIT 16
 #endif
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 #include "myutil.h"
 #include "stringutil.h"
 
+#ifdef __cplusplus
+}
+#endif
+
 SQLRETURN SQL_API MySQLColAttribute(SQLHSTMT hstmt, SQLUSMALLINT column,
                                     SQLUSMALLINT attrib, SQLCHAR **char_attr,
                                     SQLLEN *num_attr);
