@@ -1,18 +1,31 @@
-# Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
+# 
+# This program is free software; you can redistribute it and/or modify 
+# it under the terms of the GNU General Public License, version 2.0, as 
+# published by the Free Software Foundation. 
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; see the file COPYING. If not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston
-# MA  02110-1301  USA.
+# This program is also distributed with certain software (including 
+# but not limited to OpenSSL) that is licensed under separate terms, 
+# as designated in a particular file or component or in included license 
+# documentation. The authors of MySQL hereby grant you an 
+# additional permission to link the program and your derivative works 
+# with the separately licensed software that they have included with 
+# MySQL. 
+# 
+# Without limiting anything contained in the foregoing, this file, 
+# which is part of MySQL Connector/ODBC, is also subject to the 
+# Universal FOSS Exception, version 1.0, a copy of which can be found at 
+# http://oss.oracle.com/licenses/universal-foss-exception. 
+# 
+# This program is distributed in the hope that it will be useful, but 
+# WITHOUT ANY WARRANTY; without even the implied warranty of 
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+# See the GNU General Public License, version 2.0, for more details. 
+# 
+# You should have received a copy of the GNU General Public License 
+# along with this program; if not, write to the Free Software Foundation, Inc., 
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+
 
 ##############################################################################
 #
@@ -24,11 +37,11 @@
 
 %if 0%{?commercial}
 %global license_type	Commercial
-%global license_files	LICENSE.mysql
+%global license_files	LICENSE.txt
 %global product_suffix	-commercial
 %else
 %global license_type	GPLv2
-%global license_files	COPYING
+%global license_files	LICENSE.txt
 %endif
 
 # Use rpmbuild -ba --define 'shared_mysqlclient 1' ... to build shared
@@ -83,12 +96,10 @@ mysql-connector-odbc @CONNECTOR_BASE_VERSION@ is an enhanced version of mysql-co
 The driver comes in 2 flavours - ANSI and Unicode and commonly referred to as
 'MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver' or 'MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver' respectively.
 
-The MySQL software has Dual Licensing, which means you can use the
-MySQL software free of charge under the GNU General Public License
-(http://www.gnu.org/licenses/). You can also purchase commercial MySQL
-licenses from %{mysql_vendor} if you do not wish to be bound by the
-terms of the GPL. See the chapter "Licensing and Support" in the
-manual for further info.
+License information can be found in the %{license_files} file. 
+This distribution may include materials developed by third parties.
+For license and attribution notices for these materials, please refer
+to the %{license_files} file.
 
 The MySQL web site (http://www.mysql.com/) provides the latest news
 and information about the MySQL software. Also please see the
@@ -159,10 +170,10 @@ rm -rf %{buildroot}
 %install
 pushd release
 make DESTDIR=%{buildroot} install VERBOSE=1
-rm -vf  %{buildroot}%{_prefix}/{ChangeLog,README*,LICENSE.*,COPYING,INSTALL*,Licenses_for_Third-Party_Components.txt}
-mkdir -p %{buildroot}%{_datadir}/mysql-connector-odbc
-mv %{buildroot}%{_prefix}/test %{buildroot}%{_datadir}/mysql-connector-odbc/
-mv bin/dltest %{buildroot}%{_bindir}/
+rm -vf  %{buildroot}%{_prefix}/{ChangeLog,README*,LICENSE*.*}
+mkdir -p %{buildroot}%{_libdir}/mysql-connector-odbc
+mv %{buildroot}%{_prefix}/test %{buildroot}%{_libdir}/mysql-connector-odbc/
+mv bin/dltest                  %{buildroot}%{_libdir}/mysql-connector-odbc/
 popd
 
 # ----------------------------------------------------------------------
@@ -219,7 +230,7 @@ fi
 %{_libdir}/libmyodbc5w.so
 %{_libdir}/libmyodbc5a.so
 %doc %{license_files}
-%doc ChangeLog README README.debug INSTALL Licenses_for_Third-Party_Components.txt
+%doc ChangeLog README.txt
 
 %if 0%{?odbc_gui}
 %files setup
@@ -228,7 +239,7 @@ fi
 %endif
 
 %files test
-%attr(-, root, root) %{_datadir}/mysql-connector-odbc/test
-%attr(755, root, root) %{_bindir}/dltest
+%attr(-, root, root)   %{_libdir}/mysql-connector-odbc/test
+%attr(755, root, root) %{_libdir}/mysql-connector-odbc/dltest
 
 ##############################################################################
