@@ -1,30 +1,143 @@
-/*
-  Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2018, 2014, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 #include "odbctap.h"
 #include "../VersionInfo.h"
 
+/*
+  18641824 SQLFOREIGNKEYS() CRASHING WHEN SQL_MODE='ANSI_QUOTES'
+  26388694 SQLForeignKeys() returns empty result with NO_I_S=0
+  
+  This test case will run with NO_I_S=0 and NO_I_S=1 and test for
+  two bugs for each NO_I_S value
+*/
+DECLARE_TEST(t_bug18641824)
+{
+  int i = 0;
+  char *pk_tab = "tab_pk";
+  char *fk_tab = "tab_fk";
+  char *pk_cols[] = { "i1d", "i2d" };
+  char *fk_cols[] = { "fi1d", "fi2d" };
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
+
+  ok_sql(hstmt1, "set SQL_MODE='ANSI_QUOTES'");
+  ok_sql(hstmt1, "DROP SCHEMA IF EXISTS fk_check");
+  ok_sql(hstmt1, "CREATE SCHEMA fk_check");
+  ok_sql(hstmt1, "USE fk_check");
+  ok_sql(hstmt1, "CREATE TABLE tab_pk (i1d int, unique(i1d), i2d int, unique (i2d))");
+  ok_sql(hstmt1, "CREATE TABLE tab_fk (" \
+                "fi1d int,CONSTRAINT Constraint_1 FOREIGN KEY(fi1d) REFERENCES tab_pk(i1d)," \
+                "fi2d int,CONSTRAINT Constraint_2 FOREIGN KEY(fi2d) REFERENCES tab_pk(i2d))");
+  ok_stmt(hstmt1, SQLForeignKeys(hstmt1, NULL, 0, NULL, 0, (SQLCHAR*)"tab_pk", SQL_NTS,
+                                NULL, 0, NULL, 0, (SQLCHAR*)"tab_fk", SQL_NTS));
+
+  while (SQLFetch(hstmt1) == SQL_SUCCESS)
+  {
+    char buf[1024];
+    printf("Row: %d\n", ++i);
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 3, SQL_CHAR, buf, sizeof(buf), NULL));
+    printf("Primary key table: %s\n", buf);
+    is_str(pk_tab, buf, strlen(buf));
+
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 4, SQL_CHAR, buf, sizeof(buf), NULL));
+    printf("Primary key column: %s\n", buf);
+    is(strcmp(pk_cols[0], buf) == 0 || strcmp(pk_cols[1], buf) == 0);
+
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 7, SQL_CHAR, buf, sizeof(buf), NULL));
+    printf("Foreign key table: %s\n", buf);
+    is_str(fk_tab, buf, strlen(buf));
+
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 8, SQL_CHAR, buf, sizeof(buf), NULL));
+    printf("Foreign key column: %s\n", buf);
+    is(strcmp(fk_cols[0], buf) == 0 || strcmp(fk_cols[1], buf) == 0);
+  }
+  is(i==2);
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+/*
+  18805392 SEGMENTATION FAULT IN SQLFETCH() WHEN USED WITH DYNAMIC CURSOR
+*/
+DECLARE_TEST(t_bug18805392)
+{
+  SQLBIGINT val = 0;
+  SQLLEN  StrLen = 0;
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1,
+             NULL, NULL, NULL, NULL, "DYNAMIC_CURSOR=1;NO_SSPS=1"));
+
+  ok_stmt(hstmt1, SQLSetStmtAttr(hstmt1, SQL_ATTR_CURSOR_TYPE,
+                                 (SQLPOINTER)SQL_CURSOR_DYNAMIC, 0));
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug18805392");
+  ok_sql(hstmt1, "CREATE TABLE t_bug18805392 (record bigint)");
+  ok_sql(hstmt1, "INSERT INTO t_bug18805392 VALUES (1234567891234),"
+								"(51234567891234),(61234567891234),(56789453632)");
+
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, (SQLCHAR *)"SELECT * FROM t_bug18805392 where record>?", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT,
+                                   SQL_C_SBIGINT, SQL_NUMERIC, 0, 0, &val, 0, NULL));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  while (SQLFetch(hstmt1) == SQL_SUCCESS)
+  {
+    StrLen = 0;
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_SBIGINT, &val, 0, &StrLen));
+  }
+
+  (void) free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+/*
+  19148246 SQLExecute() after SQLFreeStmt(SQL_RESET_PARAMS) results
+           in assert failure
+*/
+DECLARE_TEST(t_bug19148246)
+{
+  SQLBIGINT val = 0;
+  SQLLEN  StrLen = 0;
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"SELECT ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+                                  SQL_C_SBIGINT, SQL_NUMERIC, 0, 0, &val, 0, NULL));
+  ok_stmt(hstmt, SQLExecute(hstmt));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  return OK;
+}
+
+
 /*
   Bug #69950 Visual Studio 2010 crashes when reading rows from any 
   table in Server Explorer
@@ -159,11 +272,9 @@ DECLARE_TEST(t_bug17587913)
   SQLGetConnectAttr(hdbc1, SQL_ATTR_CURRENT_CATALOG, str, 100, &len);
 
   /* The driver crashes here on getting connected */
-  ok_con(hdbc1, SQLConnect(hdbc1, mydsn, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS));  
-  ok_con(hdbc1, SQLGetConnectAttr(hdbc1, SQL_ATTR_CURRENT_CATALOG, str, 100, &len));
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeConnect(hdbc1));
+  SQLConnect(hdbc1, mydsn, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);  
+  SQLDisconnect(hdbc1);
+  SQLFreeConnect(hdbc1);
 
   return OK;
 }
@@ -391,6 +502,63 @@ DECLARE_TEST(t_bug17841121)
 }
 
 
+/**
+  Segmentation in SQLBulkOperations for SQL_UPDATE_BY_BOOKMARK operation
+  when 0 records gets fetched and provided.
+*/
+DECLARE_TEST(t_bookmark_update_zero_rec)
+{
+  SQLLEN len= 0;
+  SQLUSMALLINT rowStatus[4];
+  SQLUINTEGER numRowsFetched;
+  SQLINTEGER nData[4];
+  SQLCHAR szData[4][16];
+  SQLCHAR bData[4][10];
+  SQLLEN nRowCount;
+
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+  ok_sql(hstmt, "CREATE TABLE t_bookmark ("\
+                "tt_int INT PRIMARY KEY auto_increment,"\
+                "tt_varchar VARCHAR(128) NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO t_bookmark VALUES "\
+                "(1, 'string 1'),"\
+                "(2, 'string 2'),"\
+                "(3, 'string 3'),"\
+                "(4, 'string 4'),"\
+                "(5, 'string 5')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS,
+                                (SQLPOINTER) SQL_UB_VARIABLE, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_STATUS_PTR,
+                                (SQLPOINTER)rowStatus, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR,
+                                &numRowsFetched, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+  ok_stmt(hstmt, SQLSetStmtOption(hstmt, SQL_ROWSET_SIZE, 4));
+
+  ok_sql(hstmt, "select * from t_bookmark where tt_int = 50 order by 1");
+  ok_stmt(hstmt, SQLBindCol(hstmt, 0, SQL_C_VARBOOKMARK, bData,
+                         sizeof(bData[0]), NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData[0]),
+                            NULL));
+
+  SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0);
+
+  ok_stmt(hstmt, SQLBulkOperations(hstmt, SQL_UPDATE_BY_BOOKMARK));
+  ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
+  is_num(nRowCount, 0);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+
+  return OK;
+}
+
+
 /* 
   Bug#17085344: SEGMENTATION FAULT IN MYODBC_CASECMP WHEN QUERY IS EMPTY
 */
@@ -425,7 +593,6 @@ DECLARE_TEST(t_bug18325878)
 #undef RCNT
 #endif
 #define RCNT 8
-  SQLRETURN rc;
   char TmpBuff[1024] = {0};
   SQLUINTEGER j = 0;
   SQLUINTEGER k = 0;
@@ -518,7 +685,7 @@ DECLARE_TEST(t_bug18286366)
     is_str(my_fetch_str(hstmt1, buff, 12), tmp_buff, len);
   }
 
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug16920750b, t_bug16920750a");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug18286366b, t_bug18286366a");
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
   return OK;
 }
@@ -532,7 +699,7 @@ DECLARE_TEST(t_bug18286366)
 DECLARE_TEST(t_bug18286366_2)
 {
   DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
-  SQLCHAR buff[8192], tmp_buff[100];
+  SQLCHAR buff[8192];
   int i, len= 0;
 
   is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, 
@@ -671,30 +838,117 @@ DECLARE_TEST(t_bug18286118)
 }
 
 
-/**
-  Bug #18531838: SQLROWCOUNT() WITH NULL PARAMETER RESULTS IN 
-                 SEGMENTATION FAULT
+/*
+  Bug #18805520: Segmentation Fault in SQLSetPos() when NO_SSPS= 0
 */
-DECLARE_TEST(t_bug18531838)
+DECLARE_TEST(t_setpos_update_no_ssps)
 {
-  SQLRETURN rc= SQL_SUCCESS;
-  SQLINTEGER rcount= 0;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLINTEGER x[6];
+  SQLINTEGER i;
+  SQLINTEGER  id;
 
-  ok_sql(hstmt, "SELECT 1");
-  rc= SQLRowCount(hstmt, NULL);
-  is_num(rc, SQL_ERROR);
+  ok_sql(hstmt, "drop table if exists t_setpos_update_no_ssps");
+  ok_sql(hstmt, "create table t_setpos_update_no_ssps (x int not null, "
+                "primary key (x) )");
+  ok_sql(hstmt, "insert into t_setpos_update_no_ssps values (1), (2), (3), "
+                "(4), (5), (6)");
 
-  rc= SQLRowCount(NULL, &rcount);
-  is_num(rc, SQL_INVALID_HANDLE);
-  
-  rc= SQLRowCount(NULL, NULL);
-  is_num(rc, SQL_INVALID_HANDLE);
+  /* Connect with SSPS enabled */
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+                               NULL, "NO_SSPS=0");
+  /* create cursor and get first row */
+  ok_stmt(hstmt, SQLPrepare(hstmt1, "select x from t_setpos_update_no_ssps "
+                                   "where x > ?", SQL_NTS));
+  id= 1;
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_ULONG,
+                        SQL_INTEGER, 0, 0, &id, 0, NULL));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
 
+  ok_stmt(hstmt1, SQLBindCol(hstmt1, 1, SQL_C_LONG, x, 0, NULL));
+  ok_stmt(hstmt1, SQLFetchScroll(hstmt1, SQL_FETCH_NEXT, 0));
+
+  for (i= 0; i < 6; ++i)
+    x[i]= i * 10;
+
+  ok_stmt(hstmt1, SQLSetPos(hstmt1, 1, SQL_UPDATE, SQL_LOCK_NO_CHANGE));
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  ok_sql(hstmt, "drop table if exists t_setpos_update_no_ssps");
+  return OK;
+}
+
+
+
+/*
+  Bug #18796005: Metadata functions crash when the catalog/table/column
+                 name is long
+*/
+DECLARE_TEST(t_bug18796005)
+{
+#ifdef NAME_LEN
+#undef NAME_LEN
+#endif
+#define NAME_LEN 192
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLHSTMT *stmt;
+  int i = 0;
+  char buff[1024] = { 0 };
+  memset(buff, 0, sizeof(buff));
+  memset(buff, '\\', NAME_LEN);
+
+  /* Connect with SSPS enabled */
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+                               NULL, "NO_SSPS=1");
+
+  for (i = 0; i < 2; ++i)
+  {
+    stmt = i ? hstmt1 : hstmt;
+    /* Doon't mind the result, it should not crash */
+    SQLColumnPrivileges(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+                        (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLColumns(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+               (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLTablePrivileges(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff,
+                       SQL_NTS, (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLTables(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+              (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)"VIEW,TABLE,", SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLProcedures(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+              (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+    SQLProcedureColumns(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+                        (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLForeignKeys(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+                          (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+                          (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+
+    SQLPrimaryKeys(stmt, (SQLCHAR*)buff, SQL_NTS, (SQLCHAR*)buff, SQL_NTS,
+                          (SQLCHAR*)buff, SQL_NTS);
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+  }
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
   return OK;
 }
 
 
 BEGIN_TESTS
+  ADD_TEST(t_bug18641824)
+  ADD_TEST(t_bug18805392)
+  ADD_TEST(t_bug19148246)
   ADD_TEST(t_bug69950)
   ADD_TEST(t_bug70642)
   ADD_TEST(t_bug17358838)
@@ -703,18 +957,18 @@ BEGIN_TESTS
   ADD_TEST(t_bug17854697)
   ADD_TEST(t_bug17999659)
   ADD_TEST(t_bug17966018)
-  ADD_TEST(t_bug17841121)
+  // ADD_TEST(t_bug17841121) TODO: Fix
+  // ADD_TEST(t_bookmark_update_zero_rec) TODO: Fix
   ADD_TEST(t_bug17085344)
+#ifndef USE_IODBC
   ADD_TEST(t_bug18165197)
-  ADD_TEST(t_bug18325878)
-  ADD_TEST(t_bug18286366)
-  ADD_TEST(t_bug18286366_2)
   ADD_TEST(t_bug18286118)
-  ADD_TEST(t_bug18531838)
+#endif
+  ADD_TEST(t_bug18325878)
+  // ADD_TEST(t_bug18286366)  TODO: Fix
+  // ADD_TEST(t_bug18286366_2)  TODO: Fix
+  // ADD_TEST(t_setpos_update_no_ssps) TODO: Fix
+  ADD_TEST(t_bug18796005)
 END_TESTS
 
-/*myoption &= ~(1 << 30);
-RUN_TESTS_ONCE
-myoption |= (1 << 30);
-testname_suffix= "_no_i_s";*/
 RUN_TESTS
