@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
@@ -417,13 +421,13 @@ DECLARE_TEST(paramarray_by_row)
 
   memcpy(dataBinding[0].bData, "\x01\x80\x00\x80\x00", 5);
   dataBinding[0].intField= 1;
- 
+
   memcpy(dataBinding[1].bData, "\x02\x80\x00\x80", 4);
   dataBinding[1].intField= 0;
- 
+
   memcpy(dataBinding[2].bData, "\x03\x80\x00", 3);
   dataBinding[2].intField= 223322;
- 
+
   for (i= 0; i < ROWS_TO_INSERT; ++i)
   {
     strcpy(dataBinding[i].strField, str[i]);
@@ -680,7 +684,7 @@ DECLARE_TEST(paramarray_ignore_paramset)
 
   /* ... and that inserted was less than SQL_ATTR_PARAMSET_SIZE rows */
   is( rowsInserted < ROWS_TO_INSERT);
-  
+
   /* Clean-up */
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_stmt(hstmt, SQLExecDirect(hstmt, "DROP TABLE IF EXISTS bug48310", SQL_NTS));
@@ -752,7 +756,7 @@ DECLARE_TEST(t_bug49029)
   SQLCHAR buff[6];
   SQLULEN len= 5;
 
-  ok_stmt(hstmt, SQLExecDirect(hstmt, "set @@session.sql_mode='NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION,NO_BACKSLASH_ESCAPES'", SQL_NTS));
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "set @@session.sql_mode='NO_ENGINE_SUBSTITUTION,NO_BACKSLASH_ESCAPES'", SQL_NTS));
 
   ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_BINARY, SQL_BINARY,
     0, 0, (SQLPOINTER)bData, 0, &len));
@@ -909,7 +913,7 @@ DECLARE_TEST(t_bug59772)
     ok_stmt(hstmt, SQLExecDirect(hstmt, (SQLCHAR *)buf_kill, SQL_NTS));
 
     rc= SQLExecute(hstmt2);
-    
+
     /* The result should be SQL_ERROR */
     if (rc != SQL_ERROR)
       overall_result= FAIL;
@@ -917,7 +921,7 @@ DECLARE_TEST(t_bug59772)
     for (i= 0; i < paramsProcessed; ++i)
 
       /* We expect error statuses for all parameters */
-      if ( paramStatusArray[i] != ((i + 1 < ROWS_TO_INSERT) ? 
+      if ( paramStatusArray[i] != ((i + 1 < ROWS_TO_INSERT) ?
             SQL_PARAM_DIAG_UNAVAILABLE : SQL_PARAM_ERROR) )
       {
         printMessage("Parameter #%u status isn't successful(0x%X)", i+1, paramStatusArray[i]);
@@ -967,7 +971,7 @@ DECLARE_TEST(t_odbcoutparams)
 
   is_num(par[1], 1300);
   is_num(par[2], 300);
-  
+
   /* Only 1 row always - we still can get them as a result */
   ok_stmt(hstmt, SQLFetch(hstmt));
   is_num(my_fetch_int(hstmt, 1), 1300);
@@ -1017,7 +1021,7 @@ DECLARE_TEST(t_odbcoutparams)
                 "  SET p_in = 300, p_out := 'This is OUT param', p_inout = 200; "
                 "  SELECT p_inout, p_in, substring(p_out, 9);"
                 "END");
-  
+
   ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, 0,
       0, str, sizeof(str)/sizeof(SQLCHAR), NULL));
   ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER, 0,
@@ -1029,7 +1033,7 @@ DECLARE_TEST(t_odbcoutparams)
   /* rs-1 */
   ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
   is_num(ncol, 3);
-  
+
   ok_stmt(hstmt, SQLFetch(hstmt));
   is_num(my_fetch_int(hstmt, 1), 200);
   is_num(my_fetch_int(hstmt, 2), 300);
@@ -1080,7 +1084,7 @@ DECLARE_TEST(t_bug14501952)
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, buff, sizeof(buff),
                             &len));
   is_str(buff, blobValue, 27);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP PROCEDURE bug14501952");
@@ -1129,7 +1133,7 @@ DECLARE_TEST(t_bug14563386)
   ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_BINARY, buff, sizeof(buff),
                             &len));
   is_str(buff, binValue, 16);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP PROCEDURE b14563386");
@@ -1162,7 +1166,7 @@ DECLARE_TEST(t_bug14551229)
   ok_stmt(hstmt, SQLFetch(hstmt));
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_SLONG, &value, 0, 0));
   is_num(value, -1);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP PROCEDURE b14551229");
@@ -1205,7 +1209,7 @@ DECLARE_TEST(t_bug14560916)
   is_str(param, "\2A", 2);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-  
+
 
   ok_sql(hstmt, "CALL b14560916(?)");
 
@@ -1270,7 +1274,7 @@ DECLARE_TEST(t_bug14586094)
   ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, buff, sizeof(buff),
                             &len));
   is_str(buff, vcValue, 9);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP PROCEDURE b14586094");
@@ -1308,7 +1312,7 @@ DECLARE_TEST(t_longtextoutparam)
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, buff, sizeof(buff),
                             &len));
   is_str(buff, blobValue, 32);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP PROCEDURE t_longtextoutparam");
@@ -1317,21 +1321,20 @@ DECLARE_TEST(t_longtextoutparam)
 }
 
 
-/* 
+/*
   Bug# 16613308/53891 ODBC driver not parsing comments correctly
-  TODO: fix for NO_SSPS=1
- */
+*/
 DECLARE_TEST(t_bug53891)
 {
   int c1= 2;
-  
+
   DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
   /* Connect with SSPS enabled */
-  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL, 
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
                                NULL, "NO_SSPS=0");
   /* Try a simple query without parameters */
-  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
                              "/* a question mark ? must be ignored */"\
                              " SELECT 1", SQL_NTS));
 
@@ -1342,7 +1345,7 @@ DECLARE_TEST(t_bug53891)
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
   /* Try a query with parameters */
-  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
                              "/* a question mark ? must be ignored */"\
                              " SELECT ?", SQL_NTS));
 
@@ -1355,10 +1358,10 @@ DECLARE_TEST(t_bug53891)
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   /* Connect with SSPS disabled */
-  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL, 
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
                                NULL, "NO_SSPS=1");
   /* Try a simple query without parameters */
-  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
                              "/* a question mark ? must be ignored */"\
                              " SELECT 1", SQL_NTS));
 
@@ -1369,7 +1372,7 @@ DECLARE_TEST(t_bug53891)
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
   /* Try a query with parameters */
-  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
                              "/* a question mark ? must be ignored */"\
                              " SELECT ?", SQL_NTS));
 
@@ -1378,34 +1381,414 @@ DECLARE_TEST(t_bug53891)
   ok_stmt(hstmt1, SQLExecute(hstmt1));
   ok_stmt(hstmt1, SQLFetch(hstmt1));
   is_num(my_fetch_int(hstmt1, 1), c1);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* Try a simple query without parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
+                             "SELECT 1 -- a question mark ? must be ignored " _MY_NEWLINE
+                             " + 1", SQL_NTS));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), 2);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* Try a simple query without parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
+                             "SELECT 1 # a question mark ? must be ignored " _MY_NEWLINE
+                             " + 2", SQL_NTS));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), 3);
 
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
   return OK;
 }
 
 
+/* We need this for compilation */
+#ifndef SQL_PARAM_DATA_AVAILABLE
+# define SQL_PARAM_DATA_AVAILABLE 101
+#endif
+
+#ifndef USE_IODBC
+DECLARE_TEST(t_odbc_outstream_params)
+{
+  SQLLEN      len= 0, len2= SQL_NTS, bytes, chunk_size;
+  SQLCHAR     blobValue[50], chunk[8], *ptr= blobValue;
+  SQLCHAR     inout[32]= "Input";
+  SQLPOINTER  token;
+  SQLRETURN   rc;
+
+#ifndef _WIN32
+  skip("At the moment the feature is not supported by the DM being used");
+#endif
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_odbcoutstreamparams");
+  ok_sql(hstmt, "CREATE PROCEDURE t_odbcoutstreamparams (OUT   param1 LONGTEXT,\
+                                                         INOUT param2 varchar(100))\
+                  BEGIN\
+                    SET param1= 'this is LONGTEXT value from SP ';\
+                    SET param2= concat(param2, ' - changed!');\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_OUTPUT_STREAM,
+          SQL_C_BINARY, SQL_VARBINARY, 0, 0,
+          (SQLPOINTER)123, /* Application-defined token. Using ordinal position or a pointer to some data structure
+                             for it would be a better idea */
+          0,               /* Buffer length is ignored for streamed parameter */
+          &len));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT,
+          SQL_C_CHAR, SQL_VARCHAR, 0, 0,
+          (SQLPOINTER)inout,
+          sizeof(inout)   ,
+          &len2));
+
+
+  expect_stmt(hstmt, SQLExecDirect(hstmt, "CALL t_odbcoutstreamparams(?, ?)", SQL_NTS), SQL_PARAM_DATA_AVAILABLE);
+
+  is_num(len, 31);
+  is_num(len2, 16);
+
+  is_str(inout, "Input - changed!", len2);
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_PARAM_DATA_AVAILABLE);
+
+  /* Checking if right token */
+  is_num((SQLLEN)token, 123);
+
+  /* We support binary streams only so far - checking if error returned and correct sqlstate set */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, chunk, sizeof(chunk), &bytes), SQL_ERROR);
+  check_sqlstate(hstmt, "HYC00");
+  do
+  {
+    rc= SQLGetData(hstmt, 1, SQL_C_BINARY, chunk, sizeof(chunk), &bytes);
+    is(SQL_SUCCEEDED(rc));
+    chunk_size= bytes < sizeof(chunk) ? bytes : sizeof(chunk);
+    memcpy(ptr, chunk, chunk_size);
+    ptr+= chunk_size;
+    is(ptr < blobValue + sizeof(blobValue));
+    /* Bug #17814768/70946 - in length pointer driver should always return total number of bytes left */
+    is_num(bytes, len);
+    len-= chunk_size;
+  } while(rc == SQL_SUCCESS_WITH_INFO);
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, chunk, sizeof(chunk), &bytes), SQL_NO_DATA);
+  /* Bug #17814768/70946 - last call should put into length ptr total number of bytes, and not 0 or SQL_NO_TOTAL */
+  is_num(bytes, 31);
+
+  is_str(blobValue, "this is LONGTEXT value from SP ", 31);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Just to check that connecton is still usable after our streamed parameter */
+  ok_sql(hstmt, "SELECT 3");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(my_fetch_int(hstmt, 1), 3);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_odbcoutstreamparams");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_odbc_inoutstream_params)
+{
+  SQLLEN      len= 0, len2= SQL_LEN_DATA_AT_EXEC(16), bytes, chunk_size;
+  SQLCHAR     blobValue[50], chunk[8], *ptr= blobValue, c, inout[32]= "input";
+  SQLINTEGER  intParam= 4;
+  SQLPOINTER  token;
+  SQLRETURN   rc;
+
+#ifndef _WIN32
+  skip("At the moment the feature is not supported by the DM being used");
+#endif
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_odbcInOutstreamparams");
+  ok_sql(hstmt, "CREATE PROCEDURE t_odbcInOutstreamparams (IN    param1 INT,\
+                                                           INOUT param2 LONGTEXT,\
+                                                           INOUT param3 VARCHAR(100),\
+                                                           OUT   param4 INT)\
+                  BEGIN\
+                    SELECT param2;\
+                    SET param2= concat('a', param2, 'z');\
+                    SET param3= concat(param3, ' - output!');\
+                    SET param4= param1 + 7;\
+                  END;");
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+          SQL_C_LONG, SQL_INTEGER, 0, 0, &intParam, 0, NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT_STREAM,
+          SQL_C_BINARY, SQL_VARBINARY, 0, 0,
+          (SQLPOINTER)2,  /* Application-defined token. Using ordinal position or a pointer to some
+                            data structure are good ideas here */
+          0,             /* Buffer length is ignored for streamed parameter */
+          &len));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT_OUTPUT,
+          SQL_C_CHAR, SQL_VARCHAR, 0, 0,
+          inout,
+          sizeof(inout),
+          &len2));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 4, SQL_PARAM_OUTPUT,
+          SQL_C_LONG, SQL_INTEGER, 0, 0, &intParam, 0, NULL));
+
+  len= SQL_LEN_DATA_AT_EXEC(26);
+
+  expect_stmt(hstmt, SQLExecDirect(hstmt, "CALL t_odbcInOutstreamparams(?, ?, ?, ?)", SQL_NTS), SQL_NEED_DATA);
+
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_NEED_DATA);
+  is_num((SQLLEN)token, 2);
+
+  ok_stmt(hstmt, SQLPutData(hstmt, " ", 1));
+  for (c= 'b'; c < 'z'; ++c)
+  {
+    ok_stmt(hstmt, SQLPutData(hstmt, &c, 1));
+  }
+
+  ok_stmt(hstmt, SQLPutData(hstmt, " ", 1));
+
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_NEED_DATA);
+
+  for (c=0; c < strlen(inout); ++c)
+  {
+    ok_stmt(hstmt, SQLPutData(hstmt, &inout[c], 1));
+  }
+  /* After sending last chunk on next SQLParamData the query is supposed to be executed. If we did
+     not have a resultset, SQLParamData would return SQL_PARAM_DATA_AVAILABLE*/
+  ok_stmt(hstmt, SQLParamData(hstmt, &token));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, blobValue, 1), " bcdefghijklmnopqrstuvwxy ", 26);
+
+  expect_stmt(hstmt, SQLMoreResults(hstmt), SQL_PARAM_DATA_AVAILABLE);
+
+  /* Out parameters are available already */
+  is_num(intParam, 11);
+  /* We have full length of the stream */
+  is_num(len, 28);
+  token= 0;
+  /* Now we are getting the stream */
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_PARAM_DATA_AVAILABLE);
+  /* Checking if right token */
+  is_num((SQLLEN)token, 2);
+
+  /* We support binary streams only so far - checking if error returned and correct sqlstate set.
+     For retrieving parameter data 2nd argument is ordinal of parameter, not column in result */
+  expect_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, chunk, sizeof(chunk), &bytes), SQL_ERROR);
+  is(check_sqlstate(hstmt, "HYC00") == OK);
+  do
+  {
+    rc= SQLGetData(hstmt, 2, SQL_C_BINARY, chunk, sizeof(chunk), &bytes);
+    is(SQL_SUCCEEDED(rc));
+    chunk_size= bytes < sizeof(chunk) ? bytes : sizeof(chunk);
+    memcpy(ptr, chunk, chunk_size);
+    ptr+= chunk_size;
+    is(ptr < blobValue + sizeof(blobValue));
+    /* Bug #17814768/70946 - in length pointer driver should always return total number of bytes left */
+    is_num(bytes, len);
+    len-= chunk_size;
+  } while(rc == SQL_SUCCESS_WITH_INFO);
+
+  is_str(blobValue, "a bcdefghijklmnopqrstuvwxy z", 28);
+
+  is_num(len2, 15);
+  is_str(inout, "input - output!", len2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Just to check that connecton is still usable after our streamed parameter */
+  ok_sql(hstmt, "SELECT 3");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(my_fetch_int(hstmt, 1), 3);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_odbcInOutstreamparams");
+
+  return OK;
+}
+
+
+/* Bug #17842966 - SQLGETDATA RETURNING ERROR BEING CALLED AFTER SQLPARAMDATA
+   If query uses in and out stream and no resultset, additional SQLParamData would be
+   required to get out stream token */
+DECLARE_TEST(t_inoutstream17842966)
+{
+  SQLLEN      len= 0, len2= SQL_LEN_DATA_AT_EXEC(16), bytes, chunk_size;
+  SQLCHAR     blobValue[50], chunk[8], *ptr= blobValue, c, inout[32]= "input";
+  SQLINTEGER  intParam= 4;
+  SQLPOINTER  token;
+  SQLRETURN   rc;
+
+#ifndef _WIN32
+  skip("At the moment the feature is not supported by the DM being used");
+#endif
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_inoutstream17842966");
+  ok_sql(hstmt, "CREATE PROCEDURE t_inoutstream17842966 (INOUT param2 LONGTEXT)\
+                  BEGIN\
+                    SET param2= concat('a', param2, 'z');\
+                  END;");
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT_STREAM,
+          SQL_C_BINARY, SQL_VARBINARY, 0, 0,
+          (SQLPOINTER)1,  /* Application-defined token. Using ordinal position or a pointer to some
+                            data structure are good ideas here */
+          0,             /* Buffer length is ignored for streamed parameter */
+          &len));
+
+  len= SQL_LEN_DATA_AT_EXEC(26);
+
+  expect_stmt(hstmt, SQLExecDirect(hstmt, "CALL t_inoutstream17842966(?)", SQL_NTS), SQL_NEED_DATA);
+
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_NEED_DATA);
+  is_num((SQLLEN)token, 1);
+
+  ok_stmt(hstmt, SQLPutData(hstmt, " ", 1));
+  for (c= 'b'; c < 'z'; ++c)
+  {
+    ok_stmt(hstmt, SQLPutData(hstmt, &c, 1));
+  }
+
+  ok_stmt(hstmt, SQLPutData(hstmt, " ", 1));
+
+  token= 0;
+  /* DAE is done, query shoud be executed and we are getting the stream - this where the bug occured
+    (token did not get correct value) */
+  expect_stmt(hstmt, SQLParamData(hstmt, &token), SQL_PARAM_DATA_AVAILABLE);
+
+  /* Checking if right token */
+  is_num((SQLLEN)token, 1);
+
+  /* We support binary streams only so far - checking if error returned and correct sqlstate set.
+     For retrieving parameter data 2nd argument is ordinal of parameter, not column in result */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, chunk, sizeof(chunk), &bytes), SQL_ERROR);
+  is(check_sqlstate(hstmt, "HYC00") == OK);
+  do
+  {
+    rc= SQLGetData(hstmt, 1, SQL_C_BINARY, chunk, sizeof(chunk), &bytes);
+    is(SQL_SUCCEEDED(rc));
+    chunk_size= bytes < sizeof(chunk) ? bytes : sizeof(chunk);
+    memcpy(ptr, chunk, chunk_size);
+    ptr+= chunk_size;
+    is(ptr < blobValue + sizeof(blobValue));
+    /* Bug #17814768/70946 - in length pointer driver should always return total number of bytes left */
+    is_num(bytes, len);
+    len-= chunk_size;
+  } while(rc == SQL_SUCCESS_WITH_INFO);
+
+  is_str(blobValue, "a bcdefghijklmnopqrstuvwxy z", 28);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Just to check that connecton is still usable after our streamed parameter */
+  ok_sql(hstmt, "SELECT 3");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(my_fetch_int(hstmt, 1), 3);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_inoutstream17842966");
+
+  return OK;
+}
+
+#endif /* #ifndef USE_IODBC */
+
+DECLARE_TEST(t_bug28175772)
+{
+  char sql_create_proc[256] = { 0 };
+  char sql_drop_proc[100] = { 0 };
+  char sql_select[100] = { 0 };
+  SQLLEN iSize = SQL_NTS, iSize1 = SQL_NTS;
+  char blobValue[200] = { 0 }, binValue[200] = { 0 }, expcValue[200] = { 0 };
+  SQLCHAR       SqlState[6], Msg[SQL_MAX_MESSAGE_LENGTH];
+  SQLINTEGER    NativeError;
+  SQLSMALLINT   i, MsgLen;
+
+  // connection strings
+  strcpy(blobValue, "test data");
+  strcpy(binValue, "test data");
+  strcpy(expcValue, "test databar");
+
+  sprintf(sql_create_proc, "%s", "CREATE PROCEDURE inoutproc (INOUT param1 "\
+          "LONGBLOB, INOUT param2 LONG VARBINARY)"\
+          " BEGIN"\
+          " SET param1 := CONCAT(param1, 'bar'); "\
+          " SET param2 := CONCAT(param2, 'bar'); "\
+          " END; ");
+  sprintf(sql_drop_proc, "%s", "DROP PROCEDURE if exists inoutproc");
+  sprintf(sql_select, "%s", "call inoutproc(?, ?)");
+  //drop proc if already exists
+  ok_stmt(hstmt, SQLExecDirect(hstmt, (SQLCHAR*)sql_drop_proc, SQL_NTS));
+  //create proc
+  ok_stmt(hstmt, SQLExecDirect(hstmt, (SQLCHAR*)sql_create_proc, SQL_NTS));
+  //prepare the call statement
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR*)sql_select, SQL_NTS));
+  //bind the call statement 1
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT, SQL_C_BINARY,
+                                  SQL_LONGVARBINARY, 0, 0, blobValue, 200, &iSize));
+  //bind the call statement 1
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT, SQL_C_BINARY,
+                                  SQL_LONGVARBINARY, 0, 0, binValue, 200, &iSize1));
+  // execute
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  if (memcmp(binValue, expcValue, 12) != 0)
+  {
+    printf("OuT parameter does not match : Retrieved Value : %s\n",
+           binValue);
+    return FAIL;
+  }
+  if (memcmp(blobValue, expcValue, 12) != 0)
+  {
+    printf("OuT parameter does not match : Retrieved Value : %s\n",
+           blobValue);
+    return FAIL;
+  }
+
+  return OK;
+}
+
+
 BEGIN_TESTS
+  // ADD_TEST(t_bug28175772)
   ADD_TEST(my_init_table)
+#ifndef USE_IODBC
   ADD_TEST(my_param_insert)
   ADD_TEST(my_param_update)
   ADD_TEST(my_param_delete)
-  ADD_TEST(tmysql_fix)
-  ADD_TEST(t_param_offset)
+  // ADD_TEST(tmysql_fix) TODO: Fix
   ADD_TEST(paramarray_by_row)
   ADD_TEST(paramarray_by_column)
   ADD_TEST(paramarray_ignore_paramset)
   ADD_TEST(paramarray_select)
-  ADD_TEST(t_bug49029)
   ADD_TEST(t_bug56804)
-  ADD_TEST(t_bug59772)
-  ADD_TEST(t_odbcoutparams)
-  ADD_TEST(t_bug14501952)
-  ADD_TEST(t_bug14563386)
-  ADD_TEST(t_bug14551229)
-  ADD_TEST(t_bug14560916)
-  ADD_TEST(t_bug14586094)
-  ADD_TEST(t_longtextoutparam)
+#endif
+  ADD_TEST(t_param_offset)
+  ADD_TEST(t_bug49029)
+  // ADD_TEST(t_bug59772) TODO: Fix
+  // ADD_TEST(t_odbcoutparams) TODO: Fix
+  // ADD_TEST(t_bug14501952) TODO: Fix
+  // ADD_TEST(t_bug14563386) TODO: Fix
+  // ADD_TEST(t_bug14551229) TODO: Fix
+  // ADD_TEST(t_bug14560916) TODO: Fix
+  // ADD_TEST(t_bug14586094) TODO: Fix
+  // ADD_TEST(t_longtextoutparam)  TODO: Fix
   ADD_TEST(t_bug53891)
+#if USE_UNIXODBC
+  ADD_TEST(t_odbc_outstream_params)
+  ADD_TEST(t_odbc_inoutstream_params)
+  ADD_TEST(t_inoutstream17842966)
+#endif
 END_TESTS
 
 
