@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file odbctap.h
@@ -51,22 +55,31 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef _WIN32
+/* Driver does that - applications should do that too, to have same size of structures */
+# ifndef RC_INVOKED
+#  pragma pack(push, 1)
+# endif
+#endif
+
 #include <sql.h>
 #include <sqlext.h>
 #include <odbcinst.h>
 
+#ifdef _WIN32
+# ifndef RC_INVOKED
+#  pragma pack(pop)
+# endif
+#endif
+
 /* for clock() */
 #include <time.h>
 
-
 /* Get routines for helping with Unicode conversion. */
 #define ODBCTAP
 
-typedef unsigned int UTF32;
-typedef unsigned short UTF16;
-typedef unsigned char UTF8;
-
-#include "../util/unicode_transcode.c"
+#include "mysql_version.h"
+#include "../util/unicode_transcode.h"
 
 
 void printMessage(const char *fmt, ...)
@@ -109,7 +122,7 @@ const char * wstr4output(const wchar_t *wstr)
       break;
     }
   }
-  
+
   return str;
 }
 
@@ -119,15 +132,24 @@ const char * wstr4output(const wchar_t *wstr)
 #define MYSQL_NAME_LEN 64
 #define MAX_MEM_BLOCK_ELEMENTS 100
 
-SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 5.2 Driver}";
+#ifdef __WIN__
+#define _MY_NEWLINE "\r\n"
+#else
+#define _MY_NEWLINE "\n"
+#endif
+
+SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 5.3 Driver}";
 SQLCHAR *mydsn= (SQLCHAR *)"test";
 SQLCHAR *myuid= (SQLCHAR *)"root";
 SQLCHAR *mypwd= (SQLCHAR *)"";
 SQLCHAR *mysock= NULL;
-int      myoption= 0, myport= 0;
+int      myoption= 0, myport= 0, myenable_pooling= 0;
 SQLCHAR *my_str_options= (SQLCHAR *)""; /* String for additional connection options */
 SQLCHAR *myserver= (SQLCHAR *)"localhost";
 SQLCHAR *mydb= (SQLCHAR *)"test";
+SQLCHAR *myauth= NULL;
+SQLCHAR *myplugindir= NULL;
+
 SQLCHAR *test_db= (SQLCHAR *)"client_odbc_test";
 /* Suffix is useful if a testsuite is run more than once */
 const SQLCHAR *testname_suffix="";
@@ -136,6 +158,8 @@ int     unicode_driver= -1;
 #define REQUIRES_UNICODE_DRIVER if (unicode_driver == 0) skip("This testcase is designed for Unicode drivers only")
 #define REQUIRES_ANSI_DRIVER if (unicode_driver != 0) skip("This testcase is designed for ANSI drivers only")
 
+int      init_auth_plugin= 0;
+
 #ifndef OK
 # define OK 0
 #endif
@@ -196,7 +220,8 @@ void test_timeout(int signum);
 HANDLE halarm= NULL;
 DWORD WINAPI win32_alarm(LPVOID arg)
 {
-  DWORD timeout= ((DWORD) arg) * 1000;
+  unsigned long long llarg = (unsigned long long)arg;
+  DWORD timeout= ((DWORD) llarg) * 1000;
   while (WaitForSingleObject(halarm, timeout) == WAIT_OBJECT_0);
   test_timeout(0);
   return 0;
@@ -204,13 +229,13 @@ DWORD WINAPI win32_alarm(LPVOID arg)
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL halarm= CreateEvent(NULL, FALSE, FALSE, NULL); \
                          if (do_alarms) \
-                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 60, 0, NULL); \
+                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 1200, 0, NULL); \
                          do_alarms= 0
 #define RUN_TESTS_ALARM (void) SetEvent(halarm)
 #else
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL (void)signal(SIGALRM, test_timeout)
-#define RUN_TESTS_ALARM  if (do_alarms) alarm(60)
+#define RUN_TESTS_ALARM  if (do_alarms) alarm(1200)
 #endif
 
 void mem_debug_init()
@@ -225,8 +250,8 @@ void mem_debug_init()
 #endif
 }
 
-/* 
-  Initialize garbage collector 
+/*
+  Initialize garbage collector
   counter (or position)
 */
 void mem_gc_init()
@@ -259,7 +284,7 @@ int main(int argc, char **argv) \
 { \
   SQLHENV  henv; \
   SQLHDBC  hdbc; \
-  SQLHSTMT hstmt; \
+  SQLHSTMT hstmt = NULL; \
   int      i, num_tests, failcnt= 0; \
   ENABLE_ALARMS; \
 \
@@ -277,8 +302,16 @@ int main(int argc, char **argv) \
     mypwd=  (SQLCHAR *)getenv("TEST_PASSWORD"); \
   if (getenv("TEST_SOCKET")) \
     mysock= (SQLCHAR *)getenv("TEST_SOCKET"); \
+  if (getenv("TEST_SERVER")) \
+    myserver= (SQLCHAR *)getenv("TEST_SERVER"); \
   if (getenv("TEST_PORT")) \
     myport= atoi(getenv("TEST_PORT")); \
+  if (getenv("TEST_ENABLE_POOLING")) \
+    myenable_pooling= atoi(getenv("TEST_ENABLE_POOLING")); \
+  if (getenv("TEST_DEFAULTAUTH")) \
+    myauth=  (SQLCHAR *)getenv("TEST_DEFAULTAUTH"); \
+  if (getenv("TEST_PLUGINDIR")) \
+    myplugindir=  (SQLCHAR *)getenv("TEST_PLUGINDIR"); \
 \
   if (argc > 1) \
     mydsn= (SQLCHAR *)argv[1]; \
@@ -581,7 +614,7 @@ do { \
            #a, a1, a2, __FILE__, __LINE__); \
     return FAIL; \
   } \
-} while (0);
+} while (0)
 
 
 /* convenience macro for using statement */
@@ -638,10 +671,10 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 
 
 /**
-  Print error and diagnostic information for ODBC INSTALLER API functions 
+  Print error and diagnostic information for ODBC INSTALLER API functions
   that did not return TRUE (1)
 */
-static void print_diag_installer(BOOL is_success, const char *text, 
+static void print_diag_installer(BOOL is_success, const char *text,
                                  const char *file, int line)
 {
   if (!is_success)
@@ -838,7 +871,7 @@ int my_print_non_format_result(SQLHSTMT hstmt)
     SQLLEN      ind_strlen;
 
     rc = SQLNumResultCols(hstmt,&ncol);
-    
+
     mystmt_rows(hstmt,rc,-1);
 
     for (nIndex = 1; nIndex <= ncol; ++nIndex)
@@ -1021,7 +1054,7 @@ const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT icol)
        better/(in more easy way) test the value */
     if (nLen < 0)
     {
-      strcpy(szData, "(Null)"); 
+      strcpy(szData, "(Null)");
     }
     printMessage(" my_fetch_str: %s(%ld)", szData, nLen);
     return((const char *)szData);
@@ -1154,12 +1187,12 @@ int mydrvconnect(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt, SQLCHAR *connIn)
 }
 
 
-/* Helper function for tests to get (additional) connection 
+/* Helper function for tests to get (additional) connection
    If dsn, uid, pwd or options is null - they defualt to mydsn, myuid, mypwd
    and my_str_options, respectively.
    myoption, mysock and myport values are used. */
 int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
-                   const SQLCHAR *pwd, const SQLCHAR *db, 
+                   const SQLCHAR *pwd, const SQLCHAR *db,
                    const SQLCHAR *options)
 {
   /* Buffers have to be large enough to contain SSL options and long names */
@@ -1220,6 +1253,23 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
     strcat(connIn, options);
   }
 
+#if MYSQL_VERSION_ID >= 50507
+  if (init_auth_plugin)
+  {
+    init_auth_plugin= 0; /* reset the plugin init flag */
+    if (myauth && myauth[0])
+    {
+      strcat((char *)connIn, ";DEFAULTAUTH=");
+      strcat((char *)connIn, (char *)myauth);
+    }
+    if (myplugindir && myplugindir[0])
+    {
+      strcat((char *)connIn, ";PLUGINDIR=");
+      strcat((char *)connIn, (char *)myplugindir);
+    }
+  }
+#endif
+
   rc= SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
                        MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
 
@@ -1227,7 +1277,7 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
   {
     /* re-build and print the connection string with hidden password */
     printf("# Connection failed with the following Connection string: " \
-           "\n%s;UID=%s;PWD=*******%s%s%s;%s\n", 
+           "\n%s;UID=%s;PWD=*******%s%s%s;%s\n",
            dsn_buf, uid, socket_buf, db_buf, port_buf, options);
     return rc;
   }
@@ -1256,18 +1306,35 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
 }
 
 
-int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc, 
-                                 SQLHSTMT *hstmt,  const SQLCHAR *dsn, 
-                                 const SQLCHAR *uid, const SQLCHAR *pwd, 
+int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc,
+                                 SQLHSTMT *hstmt,  const SQLCHAR *dsn,
+                                 const SQLCHAR *uid, const SQLCHAR *pwd,
                                  const SQLCHAR *db, const SQLCHAR *options)
 {
+  if (myenable_pooling)
+  {
+    SQLSetEnvAttr(NULL, SQL_ATTR_CONNECTION_POOLING,
+                 (SQLPOINTER)SQL_CP_ONE_PER_DRIVER, 0);
+  }
+
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv));
 
   ok_env(*henv, SQLSetEnvAttr(*henv, SQL_ATTR_ODBC_VERSION,
                               (SQLPOINTER)SQL_OV_ODBC3, 0));
+  if (myenable_pooling)
+  {
+    SQLRETURN rc= SQLSetEnvAttr(*henv, SQL_ATTR_CP_MATCH, (SQLPOINTER)SQL_CP_STRICT_MATCH, 0);
+
+    if( !SQL_SUCCEEDED(rc))
+    {
+      /* No DM case? */
+      is_num(check_sqlstate_ex(*henv, SQL_HANDLE_ENV, "HYC00"), OK);
+    }
+  }
 
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
 
+  /* ok_con(*hdbc, SQLConnect(*hdbc, mydsn, strlen(mydsn), myuid, strlen(myuid), mypwd, strlen(mypwd))); */
   ok_con(*hdbc, get_connection(hdbc, dsn, uid, pwd, db, options));
 
   ok_con(*hdbc, SQLAllocHandle(SQL_HANDLE_STMT, *hdbc, hstmt));
@@ -1278,9 +1345,9 @@ int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc,
 
 int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
 {
-  return alloc_basic_handles_with_opt(henv, hdbc, hstmt, (SQLCHAR *)mydsn, 
+  return alloc_basic_handles_with_opt(henv, hdbc, hstmt, (SQLCHAR *)mydsn,
                                       (SQLCHAR *)myuid, (SQLCHAR *)mypwd,
-                                      (SQLCHAR *)mydb, 
+                                      (SQLCHAR *)mydb,
                                       (SQLCHAR *)my_str_options);
 }
 
@@ -1290,21 +1357,29 @@ int free_basic_handles(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
   /* We don't care if this succeeds, the connection may have gone away. */
   (void)SQLEndTran(SQL_HANDLE_DBC, *hdbc, SQL_COMMIT);
 
-  ok_stmt(*hstmt, SQLFreeStmt(*hstmt, SQL_DROP));
+  if(hstmt && *hstmt)
+  {
+    ok_stmt(*hstmt, SQLFreeStmt(*hstmt, SQL_DROP));
+  }
 
-  ok_con(*hdbc, SQLDisconnect(*hdbc));
+  if(hdbc && *hdbc)
+  {
+    ok_con(*hdbc, SQLDisconnect(*hdbc));
+    ok_con(*hdbc, SQLFreeConnect(*hdbc));
+  }
 
-  ok_con(*hdbc, SQLFreeConnect(*hdbc));
-
-  ok_env(*henv, SQLFreeEnv(*henv));
+  if(henv && *henv)
+  {
+    ok_env(*henv, SQLFreeEnv(*henv));
+  }
 
   return OK;
 }
 
 
-/* 
-  Allocate size bytes and return a pointer 
-  to allocated memory with storing in 
+/*
+  Allocate size bytes and return a pointer
+  to allocated memory with storing in
   garbage collector
 */
 void *gc_alloc(size_t len)
@@ -1312,8 +1387,8 @@ void *gc_alloc(size_t len)
   if (gc_blk.counter >= MAX_MEM_BLOCK_ELEMENTS || len <= 0)
   {
     printf("# GC Memory reached maximum limit counter: %d " \
-        "or length:%d less then equal to 0 in %s on line %d\n", 
-        gc_blk.counter, len, __FILE__, __LINE__); \
+        "or length:%d less then equal to 0 in %s on line %d\n",
+        gc_blk.counter, (int)len, __FILE__, __LINE__); \
     return NULL;
   }
 
@@ -1322,7 +1397,7 @@ void *gc_alloc(size_t len)
 }
 
 
-/* 
+/*
   Free allocated memory collected by
   garbage collector
 */
