@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 /**
   @file  stringutil.c
@@ -30,23 +34,17 @@
 #ifndef _STRINGUTIL_H
 #define _STRINGUTIL_H
 
+#include "../MYODBC_MYSQL.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include "../MYODBC_MYSQL.h"
-
 #include <string.h>
 #include <sql.h>
 #include <sqlext.h>
 
-#ifndef x_free
-# if MYSQL_VERSION_ID >= 50500
-#  define x_free(A) { void *tmp= (A); if (tmp) my_free((char *) tmp); }
-# else
-#  define x_free(A) { void *tmp= (A); if (tmp) my_free((char *) tmp,MYF(MY_WME+MY_FAE)); }
-# endif
-#endif
+#include "unicode_transcode.h"
 
 #define myodbc_min(a, b) ((a) < (b) ? (a) : (b))
 #define myodbc_max(a, b) ((a) > (b) ? (a) : (b))
@@ -59,11 +57,6 @@ extern "C" {
 
 #define MAX_BYTES_PER_UTF8_CP 4 /* max 4 bytes per utf8 codepoint */
 
-/* Unicode transcoding */
-typedef unsigned int UTF32;
-typedef unsigned short UTF16;
-typedef unsigned char UTF8;
-
 extern CHARSET_INFO *utf8_charset_info;
 
 
@@ -115,8 +108,13 @@ unsigned long sqlwchartoul(const SQLWCHAR *wstr, const SQLWCHAR **endptr);
 void sqlwcharfromul(SQLWCHAR *wstr, unsigned long v);
 size_t sqlwcharncat2(SQLWCHAR *dest, const SQLWCHAR *src, size_t *n);
 SQLWCHAR *sqlwcharncpy(SQLWCHAR *dest, const SQLWCHAR *src, size_t n);
+SQLWCHAR *wchar_t_as_sqlwchar(wchar_t *from, SQLWCHAR *to, size_t len);
 
 char * myodbc_strlwr(char *target, size_t len);
+SQLCHAR* sqlwchar_as_utf8_simple(SQLWCHAR *s);
+char *myodbc_stpmov(char *dst, const char *src);
+char *myodbc_ll2str(longlong val, char *dst, int radix);
+void myodbc_qsort(void *base_ptr, size_t count, size_t size, qsort_cmp cmp);
 #ifdef __cplusplus
 }
 #endif
